/***************************************************************************
                          bsdscan.cpp  -  description
                             -------------------
    begin                : Fri Nov 26 1999
    copyright            : (C) 1999,2000 by fflewddur
    email                : fflewddur@linuxfreak.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bsdscan.h"

// function get_input()
// task: prompt user for target, ports, etc.
// input: none
// output: port_s, port_f, target_ip
void get_input(char *target_ip, int args[]);

// function help()
// task: display usage info
// input: none
// output: none
void help(void);

// function arg_use()
// task: control argument usage and check for illegal combinations
// input: argc, argv
// ouput: args[], optind
void arg_use(int argc, char *argv[], int args[]);

// function err_quit()
// task: print an error message and exit the program
// input: error_str
// output: none
void err_quit(char *error_str);

// these functions are defined in scan.cpp
void scan(char *target_ip, int args[], char *argv[]);


/* -------------------------------------- */

int main(int argc, char *argv[])
{
	struct hostent *host;
	char target_ip[MAX_ADDR_LEN], temp_ip[INET_ADDRSTRLEN];
	boolean interactive; // on TRUE we prompt for input
	int args[MAX_ARGS]; // if an arg is used, it will be set to TRUE, otherwise it's FALSE

	// arg list
	//	0 - logging enabled
	//	1 - points to the argv[] string with the log file
	//	2 - wildcard detected
	//	3 - location in string of wildcard
	//	4 - use stealth mode
	//	5 - ports defined on command-line
	//	6 - lowest port
	//	7 - highest port
	//	8 - only scan common ports
	//	9 - length of delay between sent packets
	//

	// set all of args[] elements' to 0
	for (int i = 0; i <= MAX_ARGS; i++)
		args[i] = 0;

	// check for proper usage
	arg_use(argc, argv, args);

	if (args[WILDCARD] == 0)
    {
		// if the wildcard wasn't used, copy the ip from the command line into
		// target_ip, but don't let it overflow the buffer, and make sure it
		// terminates
		strncpy(target_ip, argv[optind], sizeof(target_ip) - 1);
		target_ip[sizeof(target_ip) - 1] = '\0';

		// now convert target_ip into an ip incase it was input as a domain
		// name, and put the result back into target_ip
		host = Gethostbyname(target_ip);
		Inet_ntop(AF_INET, host->h_addr_list[0], temp_ip, INET_ADDRSTRLEN);
		strncpy(target_ip, temp_ip, sizeof(target_ip));
	}
	else
	{
		// if a wildcard was specified, copy the first three octects of the ip
		// into target_ip (argv[1] was already trimmed down to three octects in
		// arg_use() )
		strncpy(target_ip, argv[optind], sizeof(target_ip) - 1);
	}

	// scanning code
	scan(target_ip, args, argv);

	return 0;
}

/* -------------------------------------- */

void help(void)
{
	printf("Usage: bsdscan [options] <target machine>\n\n");
	printf("Options:\n");
	printf("\t--common\t-c\t\tOnly scans common ports\n");
	printf("\t--delay <x>\t-d <x>\t\tWait <x> seconds between sending packets\n");
	printf("\t--device <name>\t-i <name>\tSpecify network interface\n");
	printf("\t--help\t\t-h\t\tPrints this usage information\n");
	printf("\t--log <file>\t-l <file>\tLogs results to <file>\n");
	printf("\t--ports <x> <y>\t-p <x> <y>\tScans between ports <x> and <y>\n");
	printf("\t--shuffle\t\t\tShuffles the ports\n");
	printf("\t--stealth\t-s\t\tFIN/ACK scan\n");
	printf("\t--windows\t-w\t\tScans for SMB servers\n");
	printf("\t--version\t-v\t\tDisplays version info\n");

	exit (EXIT_FAILURE);
}

/* -------------------------------------- */

void arg_use(int argc, char *argv[], int args[])
{
	int c, option_index = 0;

	// long options
	struct option long_options[] =
	{
		{"common", no_argument, 0, 'c'},
		{"delay", required_argument, 0, 'd'},
		{"device", required_argument, 0, 'i'},
		{"help", no_argument, 0, 'h'},
		{"log", required_argument, 0, 'l'},
		{"ports", required_argument, 0, 'p'},
		{"shuffle", no_argument, &args[SHUFFLE], 1},
		{"stealth", no_argument, 0, 's'},
		{"windows", no_argument, 0, 'w'},
		{"version", no_argument, 0, 'v'},
		{0, 0, 0, 0,}
	};

	while ( (c = getopt_long(argc, argv, "cd:hl:p:swv", long_options, &option_index)) != -1)
	{
		switch (c)
		{
			case 'c':
				args[COMMON] = 1;
				break;
			case 'd':
				args[DELAY] = atoi(optarg);
				break;
			case 'h':
				help();
				break;
			case 'i':
				args[DEVICE] = optind;
				break;
			case 'l':
				args[LOG] = 1;
				args[LOGFILE] = optind;
				break;
			case 'p':
				args[PORT] = 1;
				args[LOWPORT] = atoi(argv[optind]);
				args[HIGHPORT] = atoi(argv[optind + 1]);
				break;
			case 's':
				args[STEALTH] = 1;
				break;
			case 'w':
				args[WINDOW] = 1;
				args[PORT] = 1;
				args[LOWPORT] = 139;
				args[HIGHPORT] = 139;
				break;
			case 'v':
				cout << "\tBsdScan v" << VERSION << " by fflewddur <fflewddur@linuxfreak.com>\n\t";
				printf("Released %s under version 2 of the GPL\n", RELEASE_DATE);
				exit (EXIT_SUCCESS);
			case '?':
				break;
		}
	}

	// program needs a target machine :)
	if (argv[optind] == NULL)
	{
		help();
		exit(EXIT_SUCCESS);
	}

	// check for a wildcard
	for (int i = 0; i < strlen(argv[optind]); i++)
	{
		if (argv[optind][i] == '*')
		{
			args[WILDCARD] = 1;
			args[WILDCARD_LOC] = i;
			argv[optind][i] = '\0';
		}
	}

	return;
}

/* -------------------------------------- */

void err_quit(char *error_str)
{
	printf("*** Error: %s ***\n", error_str);
	exit (EXIT_FAILURE);
}

/* -------------------------------------- */