/***************************************************************************
                          scan.cpp  -  description
                             -------------------
    begin                : Fri Nov 26 1999
    copyright            : (C) 1999,2000 by fflewddur
    email                : fflewddur@linuxfreak.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bsdscan.h"
#include <time.h>
#include <fstream.h>
#include <libnet.h>
#include <sys/param.h>
#include <sys/utsname.h>

// task: perform the specified scan
void scan(char *target_ip, int args[], char *argv[]);

// task: make sure target is reachable
int probe(char *target_ip, int log, ofstream &fout);

// task: shuffle the order of the ports to be scanned
void shuffle(int port_array[], int first_port, int last_port, int common);

// task: get local ip
u_long get_ip(void);

// task: sort an array of ints
void sort(int array[], int size);

// function declarations from bsdscan.cpp
void err_quit(char *error_str);


/* --------------------------------------------- */

void scan(char *target_ip, int args[], char *argv[])
{
	struct SA_IN target;
	struct servent *service;
	int low_port, high_port, sockfd, result, total, array_size, j, raw_sock; /* j is a counter */
	char ip_str[4];
	ofstream fout(1);
	// for libnet
	u_char *packet;
	u_long source_ip;
	struct tcp_packet response;

 	// output goes to fout, which either points to stdout (1) or a logfile
	// if one was specified
	if (args[LOG])
	{
		fout.close();
		fout.open(argv[args[LOGFILE]]);
	}

	if (args[PORT]) // PORT is set to 1 in arg_use() if --port is specified
	{
		low_port = args[LOWPORT];
		high_port = args[HIGHPORT];
	}
	else
	{
		low_port = 1;
		high_port = 1024;
	}

	array_size = (high_port - low_port) + 1;
	int *port_array = new int[array_size];
	int *open_ports = new int[array_size];

	bzero(&target, sizeof(target)); // fill target address with 0's
	target.sin_family = AF_INET;
	total = 0; // total number of open ports, needed to display output correctly

	// initalize port_array to 0
	for (int j = 0; j < array_size; j++)
		port_array[j] = 0;

	// shuffle the port numbers in port_array
	if (args[SHUFFLE])
		shuffle(port_array, low_port, high_port, args[COMMON]);
	else
		for (int i = 0; i <= array_size; i++)
			port_array[i] = i + low_port;

	for (int i = 1; i < 256; i++)
	{
		j = 0;

		if (args[WILDCARD])
		{
			memset(ip_str, sizeof(ip_str), '\0'); // make sure ip_str is empty
			snprintf(ip_str, sizeof(ip_str), "%d", i); // put i into a ip_str
			target_ip[args[WILDCARD_LOC]] = '\0'; // clear target_ip after 1st three ocets
			strncat(target_ip, ip_str, sizeof(target_ip)); // add 1-255 onto target_ip for a subnet scan
		}

		Inet_pton(AF_INET, target_ip, &target.sin_addr);

		fout << "\nOpen ports on " << target_ip << " between " << low_port << " and " << high_port << ":\n\t----------------\n";

		// first make sure the target is reachable
		if (probe(target_ip, args[LOG], fout))
		{
			if (args[WILDCARD])
				continue;
			else
				exit(EXIT_FAILURE);
		}

		fout << "\tPort\tService\n";

		// actuall scanning loop
		for (i = 0; i < array_size; i++)
		{

			if (args[STEALTH])
			{

				libnet_init_packet(LIBNET_IP_H + LIBNET_TCP_H, &packet);

				if (packet == NULL)
					libnet_error(LIBNET_ERR_FATAL, "libnet_init_packet failed\n");

				sockfd = libnet_open_raw_sock(IPPROTO_RAW);
				if (sockfd == -1)
					libnet_error(LIBNET_ERR_FATAL, "libnet_open_raw_sock failed\n");

				// open a socket to catch the response
				raw_sock = socket(PF_INET, SOCK_RAW, 0);
				if (raw_sock < 0)
					printf("error opening raw socket\n");

				source_ip = get_ip();

				libnet_build_ip(LIBNET_TCP_H, IPTOS_LOWDELAY, 242, 0, 48, IPPROTO_TCP, source_ip, target.sin_addr.s_addr, NULL, 0, packet);
				libnet_build_tcp(2345, htons(port_array[i]), 0xa1d95, 0x53, TH_SYN, 1024, 0, NULL, 0, packet + LIBNET_IP_H);

				if (libnet_do_checksum(packet, IPPROTO_TCP, LIBNET_TCP_H) == -1)
					libnet_error(LIBNET_ERR_FATAL, "libnet_do_checksum failed.\n");

				if (libnet_write_ip(sockfd, packet, LIBNET_IP_H + LIBNET_TCP_H) < sizeof(packet))
					printf("I think we have a problem\n");
				else
					printf("Things look ok so far\n");

				read(raw_sock, &response, sizeof(struct tcp_packet));
				printf("passed read()\n");

			/*	if (response.tcp_hdr.th_flags == TH_ACK)
					printf("i think %d is open\n", port_array[i]);
				if (response.tcp_hdr.th_flags == TH_RST)
					printf("i think %d is closed\n", port_array[i]);
			*/

				libnet_close_raw_sock(sockfd);
				libnet_destroy_packet(&packet);
				close(raw_sock);

				fout << "stealth scanning still in the works...\n";
				exit(EXIT_SUCCESS);
			}
			else
			{
				// set up the socket and port info
				sockfd = Socket(AF_INET, SOCK_STREAM, 0);
				target.sin_port = htons(port_array[i]);

				result = connect(sockfd, (const SA *)&target, sizeof(SA));

				close (sockfd); // very important
			}

			if (!result)
			{
				open_ports[j++] = port_array[i];
			}

			usleep((unsigned long)args[DELAY] * 1000000);
		}

		for (j = 0; j < array_size; j++)
		{
			if (open_ports[j])
			{
				service = getservbyport(htons(open_ports[j]), "tcp");
				if (service == NULL)
					fout << "\t" << open_ports[j] << "\t" << "Unknown service\n";
				else
					fout << "\t" << open_ports[j] << "\t" << service->s_name << "\n";
			}
		}

		if (!args[WILDCARD]) // if we're only scanning one machine, exit the loop
			break;
	}

	// let's be tidy now
	delete [] port_array;
	port_array = NULL;
	delete [] open_ports;
	open_ports = NULL;

	fout.close();

	return;
}

/* --------------------------------------------- */

int probe(char *target_ip, int log, ofstream &fout) // try to connect to target_ip, and if there is no route, print an error and return -1
{
	struct SA_IN target;
	int sockfd, result;

	bzero(&target, sizeof(target));
	target.sin_family = AF_INET;
	target.sin_port = htons(7);
	Inet_pton(AF_INET, target_ip, &target.sin_addr);

	sockfd = Socket(AF_INET, SOCK_STREAM, 0);
	result = connect(sockfd, (const SA *)&target, sizeof(struct SA));
	if (errno == ETIMEDOUT || errno == EHOSTUNREACH || errno == ENETUNREACH)
	{
		if (log)
		{
			if (errno == ETIMEDOUT)
				fout << "Connection timed out\n";
			else
				fout << "Network unreachable\n";
		}
		else
			perror("connect");

		close(sockfd);

		return -1;
	}

	close(sockfd);

	return 0;
}

/* --------------------------------------------- */

void shuffle(int port_array[], int low_port, int high_port, int common)
{
	int rnd_port, array_size;

	array_size = (high_port - low_port) + 1;

	// if the 'common' option was selected, only scan the following ports
	if (common)
	{
		int common_ports[] = {7, 13, 19, 21, 22, 23, 25, 37, 42, 49, 53, 69, 70, 79, 80, 88, 110, 111, 113, 119, 137, 139, 143, 512, 513, 1080, -1};
		for (unsigned int i = 0; i < sizeof(common_ports) / sizeof(int); i++)
			port_array[i] = common_ports[i];
	}
	// if 'common' wasn't specified, generate a random array of the ports to scan
	else
	{
		// setup an array containing the ports to be put into the final array
		int *ordered_array = new int[array_size];

		for (int i = 0; i < array_size; i++)
		{
			//printf("top of loop %d\n", i);
			ordered_array[i] = low_port + i;
			//printf("bottom of loop %d\n", i);
		}

		srandom(time(NULL)); // seed random() with the current time

		// run through this loop until every element in port_array is filled with ports in random order
		for (int i = 0; i < array_size; )
		{
			rnd_port = random() % array_size;
			if (ordered_array[rnd_port])
			{
				port_array[i++] = ordered_array[rnd_port]; // i is incremented here to avoid skipping over ports
				ordered_array[rnd_port] = 0;
			}
		}

		delete [] ordered_array;
		ordered_array = NULL;
	}
}

/* --------------------------------------------- */

u_long get_ip(void)
{
	// the most bloated function ever
	char name[MAXHOSTNAMELEN];
	char test_str[MAXHOSTNAMELEN];
	struct hostent *local_machine;
	struct utsname local_name;
	struct SA_IN address;

	if (uname(&local_name) < 0)
	{
		perror("uname");
		exit(EXIT_FAILURE);
	}

	if ( (local_machine = gethostbyname(local_name.nodename)) == NULL)
	{
		perror("gethostbyname");
		exit(EXIT_FAILURE);
	}

	Inet_ntop(AF_INET, local_machine->h_addr_list[0], test_str, MAXHOSTNAMELEN);
	Inet_pton(AF_INET, test_str, &address.sin_addr);

	return address.sin_addr.s_addr;
}

/* --------------------------------------------- */